//--------------------------------------------------------------------------------------
// File: BasicHLSL.cpp
//
// This sample shows a simple example of the Microsoft Direct3D's High-Level 
// Shader Language (HLSL) using the Effect interface. 
//
// Copyright (c) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------
#include <DXUT.h>
#include <DXUTcamera.h>
#include <DXUTgui.h>
#include <DXUTsettingsdlg.h>
#include <SDKmisc.h>
#include "resource.h"

#include <Coef.h>
#include <Model.h>
#include <TexBank.h>
#include <Bone.h>
#include <MySprite.h>
#include <mqoobject.h>

#include <OrgWindow.h>

#define DBGH
#include <dbg.h>

using namespace OrgWinGUI;

typedef struct tag_tlelem
{
	int menuindex;
	int motionid;
}TLELEM;

typedef struct tag_modelelem
{
	CModel* modelptr;
	vector<TLELEM> tlarray;
	int motmenuindex;
	map<int, int> lineno2boneno;
	map<int, int> boneno2lineno;
}MODELELEM;

static vector<TLELEM> s_tlarray;
static int s_curmotmenuindex = -1;

static HWND		s_mainwnd = 0;
static HMENU	s_mainmenu = 0;
static HMENU	s_animmenu = 0;
static HMENU	s_modelmenu = 0;

static LPDIRECT3DDEVICE9 s_pdev = 0;

static CModel* s_model = NULL;
static CModel* s_select = NULL;
static CModel* s_bmark = NULL;
static CMySprite* s_bcircle = 0;

static int s_curboneno = -1;

static OrgWindow* s_timelineWnd = 0;
static OWP_Timeline* s_owpTimeline = 0;

static OWP_PlayerButton* s_owpPlayerButton = 0;

static OrgWindow* s_toolWnd = 0;		
static OWP_Button* s_toolCopyB = 0;
static OWP_Button* s_toolSymCopyB = 0;
static OWP_Button* s_toolCutB = 0;
static OWP_Button* s_toolPasteB = 0;
static OWP_Button* s_toolDeleteB = 0;

static OrgWindow* s_layerWnd = 0;
static OWP_LayerTable* s_owpLayerTable = 0;

static bool s_closeFlag = false;			// ItO
static bool s_closetoolFlag = false;
static bool s_closeobjFlag = false;
static bool s_copyFlag = false;			// Rs[tO
static bool s_symcopyFlag = false;
static bool s_cutFlag = false;			// JbgtO
static bool s_pasteFlag = false;			// y[XgtO
static bool s_cursorFlag = false;			// J[\ړtO
static bool s_selectFlag = false;			// L[ItO
static bool s_keyShiftFlag = false;		// L[ړtO
static bool s_deleteFlag = false;		// L[폜tO

static int s_previewFlag = 0;			// vr[tO
static bool s_befkeyFlag = false;
static bool s_nextkeyFlag = false;

static bool s_dispmw = true;
static bool s_disptool = true;
static bool s_dispobj = true;

static double s_keyShiftTime = 0.0;			// L[ړ
static list<OWP_Timeline::KeyInfo> s_copyKeyInfoList;	// Rs[ꂽL[񃊃Xg
static list<OWP_Timeline::KeyInfo> s_deletedKeyInfoList;	// 폜ꂽL[񃊃Xg
static list<OWP_Timeline::KeyInfo> s_selectKeyInfoList;	// Rs[ꂽL[񃊃Xg

static map<int, int> s_lineno2boneno;
static map<int, int> s_boneno2lineno;

static vector<MODELELEM> s_modelindex;
static MODELBOUND	s_totalmb;
static int s_curmodelmenuindex = -1;

//#define DEBUG_VS   // Uncomment this line to debug vertex shaders 
//#define DEBUG_PS   // Uncomment this line to debug pixel shaders 

#define CHARANUM	50
D3DXVECTOR3		g_vCenter;
CTexBank*	g_texbank = 0;

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------
ID3DXFont*                  g_pFont = NULL;         // Font for drawing text
ID3DXSprite*                g_pSprite = NULL;       // Sprite for batching draw text calls
bool                        g_bShowHelp = true;     // If true, it renders the UI control text
CModelViewerCamera          g_Camera;               // A model viewing camera
ID3DXEffect*                g_pEffect = NULL;       // D3DX effect interface
//ID3DXMesh*                  g_pMesh = NULL;         // Mesh object
//IDirect3DTexture9*          g_pMeshTexture = NULL;  // Mesh texture



CDXUTDialogResourceManager  g_DialogResourceManager; // manager for shared resources of dialogs
CD3DSettingsDlg             g_SettingsDlg;          // Device settings dialog
CDXUTDialog                 g_HUD;                  // manages the 3D UI
CDXUTDialog                 g_SampleUI;             // dialog for sample specific controls
bool                        g_bEnablePreshader;     // if TRUE, then D3DXSHADER_NO_PRESHADER is used when compiling the shader
D3DXMATRIXA16               g_mCenterWorld;

#define MAX_LIGHTS 3
CDXUTDirectionWidget g_LightControl[MAX_LIGHTS];
float                       g_fLightScale;
int                         g_nNumActiveLights;
int                         g_nActiveLight;

double						g_dspeed = 1.0;

D3DXHANDLE g_hRenderBoneL0 = 0;
D3DXHANDLE g_hRenderBoneL1 = 0;
D3DXHANDLE g_hRenderBoneL2 = 0;
D3DXHANDLE g_hRenderBoneL3 = 0;
D3DXHANDLE g_hRenderNoBoneL0 = 0;
D3DXHANDLE g_hRenderNoBoneL1 = 0;
D3DXHANDLE g_hRenderNoBoneL2 = 0;
D3DXHANDLE g_hRenderNoBoneL3 = 0;
D3DXHANDLE g_hRenderLine = 0;
D3DXHANDLE g_hRenderSprite = 0;

D3DXHANDLE g_hmWMatArray = 0;
D3DXHANDLE g_hmWorld = 0;
D3DXHANDLE g_hmVP = 0;
D3DXHANDLE g_hEyePos = 0;
D3DXHANDLE g_hnNumLight = 0;
D3DXHANDLE g_hLightDir = 0;
D3DXHANDLE g_hLightDiffuse = 0;
D3DXHANDLE g_hLightAmbient = 0;

D3DXHANDLE g_hdiffuse = 0;
D3DXHANDLE g_hambient = 0;
D3DXHANDLE g_hspecular = 0;
D3DXHANDLE g_hpower = 0;
D3DXHANDLE g_hemissive = 0;
D3DXHANDLE g_hMeshTexture = 0;

BYTE g_keybuf[256];

WCHAR g_basedir[ MAX_PATH ] = {0};


//--------------------------------------------------------------------------------------
// UI control IDs
//--------------------------------------------------------------------------------------
#define IDC_TOGGLEFULLSCREEN    1
#define IDC_TOGGLEREF           3
#define IDC_CHANGEDEVICE        4
#define IDC_ENABLE_PRESHADER    5
#define IDC_NUM_LIGHTS          6
#define IDC_NUM_LIGHTS_STATIC   7
#define IDC_ACTIVE_LIGHT        8
#define IDC_LIGHT_SCALE         9
#define IDC_LIGHT_SCALE_STATIC  10

#define IDC_COMBO_BONE			11
#define IDC_FK_XP				12
#define IDC_FK_XM				13
#define IDC_FK_YP				14
#define IDC_FK_YM				15
#define IDC_FK_ZP				16
#define IDC_FK_ZM				17

#define IDC_T_XP				18
#define IDC_T_XM				19
#define IDC_T_YP				20
#define IDC_T_YM				21
#define IDC_T_ZP				22
#define IDC_T_ZM				23

#define IDC_SPEED_STATIC		24
#define IDC_SPEED				25

//--------------------------------------------------------------------------------------
// Forward declarations 
//--------------------------------------------------------------------------------------
bool CALLBACK IsDeviceAcceptable( D3DCAPS9* pCaps, D3DFORMAT AdapterFormat, D3DFORMAT BackBufferFormat, bool bWindowed,
                                  void* pUserContext );
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext );
HRESULT CALLBACK OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc,
                                 void* pUserContext );
HRESULT CALLBACK OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc,
                                void* pUserContext );
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext );
void CALLBACK OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext );
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing,
                          void* pUserContext );
void CALLBACK KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext );
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );
void CALLBACK OnLostDevice( void* pUserContext );
void CALLBACK OnDestroyDevice( void* pUserContext );

void InitApp();
HRESULT LoadMesh( IDirect3DDevice9* pd3dDevice, WCHAR* strFileName, ID3DXMesh** ppMesh );
void RenderText( double fTime );



static int GetShaderHandle();
static int SetBaseDir();
static int OpenMQOFile();
static void refreshTimeline(OWP_Timeline& timeline);
static int AddBoneEul( float addx, float addy, float addz );
static int AddBoneTra( float addx, float addy, float addz );
static int DispMotionWindow();
static int DispToolWindow();
static int DispObjPanel();
static int EraseKeyList();
static int DestroyTimeLine( int dellist );
static int AddTimeLine( WCHAR* newname, int newmotid );
static int AddMotion();
static int OnAnimMenu( int selindex );
static int OnDelMotion( int delindex );
static int AddModelBound( MODELBOUND* mb, MODELBOUND* addmb );

static int OnModelMenu( int selindex );
static int OnDelModel( int delindex );
static int OnDelAllModel();
static int refreshModelPanel();


//--------------------------------------------------------------------------------------
// Entry point to the program. Initializes everything and goes into a message processing 
// loop. Idle time is used to render the scene.
//--------------------------------------------------------------------------------------
INT WINAPI wWinMain( HINSTANCE, HINSTANCE, LPWSTR, int )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

	OpenDbgFile();
//_CrtSetBreakAlloc(2806);
//_CrtSetBreakAlloc(758);

	SetBaseDir();

	
    // Set the callback functions. These functions allow DXUT to notify
    // the application about device changes, user input, and windows messages.  The 
    // callbacks are optional so you need only set callbacks for events you're interested 
    // in. However, if you don't handle the device reset/lost callbacks then the sample 
    // framework won't be able to reset your device since the application must first 
    // release all device resources before resetting.  Likewise, if you don't handle the 
    // device created/destroyed callbacks then DXUT won't be able to 
    // recreate your device resources.
    DXUTSetCallbackD3D9DeviceAcceptable( IsDeviceAcceptable );
    DXUTSetCallbackD3D9DeviceCreated( OnCreateDevice );
    DXUTSetCallbackD3D9DeviceReset( OnResetDevice );
    DXUTSetCallbackD3D9FrameRender( OnFrameRender );
    DXUTSetCallbackD3D9DeviceLost( OnLostDevice );
    DXUTSetCallbackD3D9DeviceDestroyed( OnDestroyDevice );
    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackKeyboard( KeyboardProc );
    DXUTSetCallbackFrameMove( OnFrameMove );
    DXUTSetCallbackDeviceChanging( ModifyDeviceSettings );

    // Show the cursor and clip it when in full screen
    DXUTSetCursorSettings( true, true );

    InitApp();

	s_mainmenu = LoadMenuW( (HINSTANCE)GetModuleHandle(NULL), MAKEINTRESOURCE( IDR_MENU1 ) );
	if( s_mainmenu == NULL ){
		_ASSERT( 0 );
		return 1;
	}

    // Initialize DXUT and create the desired Win32 window and Direct3D 
    // device for the application. Calling each of these functions is optional, but they
    // allow you to set several options which control the behavior of the framework.
    DXUTInit( true, true ); // Parse the command line and show msgboxes
    DXUTSetHotkeyHandling( true, true, true );
    DXUTCreateWindow( L"OpenRDB", 0, 0, s_mainmenu );
	DXUTCreateDevice( true, 640, 650 );
	s_mainwnd = DXUTGetHWND();
	_ASSERT( s_mainwnd );


	//animmenu
	HMENU motmenu;
	motmenu = GetSubMenu( s_mainmenu, 2 );
	s_animmenu = GetSubMenu( motmenu, 3 );
	_ASSERT( s_animmenu );

	HMENU mdlmenu = GetSubMenu( s_mainmenu, 3 );
	s_modelmenu = GetSubMenu( mdlmenu, 3 );


    // Pass control to DXUT for handling the message pump and 
    // dispatching render calls. DXUT will call your FrameMove 
    // and FrameRender callback when there is idle time between handling window messages.
    DXUTMainLoop();

    // Perform any application-level cleanup here. Direct3D device resources are released within the
    // appropriate callback functions and therefore don't require any cleanup code here


    return DXUTGetExitCode();
}


//--------------------------------------------------------------------------------------
// Initialize the app 
//--------------------------------------------------------------------------------------
void InitApp()
{
    g_bEnablePreshader = true;

    for( int i = 0; i < MAX_LIGHTS; i++ )
        g_LightControl[i].SetLightDirection( D3DXVECTOR3( sinf( D3DX_PI * 2 * i / MAX_LIGHTS - D3DX_PI / 6 ),
                                                          0, -cosf( D3DX_PI * 2 * i / MAX_LIGHTS - D3DX_PI / 6 ) ) );

    g_nActiveLight = 0;
    g_nNumActiveLights = 1;
    g_fLightScale = 1.0f;

    // Initialize dialogs
    g_SettingsDlg.Init( &g_DialogResourceManager );
    g_HUD.Init( &g_DialogResourceManager );
    g_SampleUI.Init( &g_DialogResourceManager );

    g_HUD.SetCallback( OnGUIEvent ); int iY = 10;
    g_HUD.AddButton( IDC_TOGGLEFULLSCREEN, L"Toggle full screen", 35, iY, 125, 22 );
    g_HUD.AddButton( IDC_TOGGLEREF, L"Toggle REF (F3)", 35, iY += 24, 125, 22 );
    g_HUD.AddButton( IDC_CHANGEDEVICE, L"Change device (F2)", 35, iY += 24, 125, 22, VK_F2 );

    g_SampleUI.SetCallback( OnGUIEvent ); iY = 0;

    WCHAR sz[100];
    swprintf_s( sz, 100, L"# Lights: %d", g_nNumActiveLights );
    g_SampleUI.AddStatic( IDC_NUM_LIGHTS_STATIC, sz, 35, iY += 24, 125, 22 );
    g_SampleUI.AddSlider( IDC_NUM_LIGHTS, 50, iY += 24, 100, 22, 1, MAX_LIGHTS, g_nNumActiveLights );

    iY += 24;
    swprintf_s( sz, 100, L"Light scale: %0.2f", g_fLightScale );
    g_SampleUI.AddStatic( IDC_LIGHT_SCALE_STATIC, sz, 35, iY += 24, 125, 22 );
    g_SampleUI.AddSlider( IDC_LIGHT_SCALE, 50, iY += 24, 100, 22, 0, 20, ( int )( g_fLightScale * 10.0f ) );

    iY += 24;
    g_SampleUI.AddButton( IDC_ACTIVE_LIGHT, L"Change active light (K)", 35, iY += 24, 125, 22, 'K' );
    g_SampleUI.AddCheckBox( IDC_ENABLE_PRESHADER, L"Enable preshaders", 35, iY += 24, 125, 22, g_bEnablePreshader );

	iY += 24;
	g_SampleUI.AddComboBox( IDC_COMBO_BONE, 35, iY += 24, 125, 22 );
	g_SampleUI.AddButton( IDC_FK_XP, L"Rot X+", 35, iY += 24, 60, 22 );
	g_SampleUI.AddButton( IDC_FK_XM, L"Rot X-", 100, iY, 60, 22 );
	g_SampleUI.AddButton( IDC_FK_YP, L"Rot Y+", 35, iY += 24, 60, 22 );
	g_SampleUI.AddButton( IDC_FK_YM, L"Rot Y-", 100, iY, 60, 22 );
	g_SampleUI.AddButton( IDC_FK_ZP, L"Rot Z+", 35, iY += 24, 60, 22 );
	g_SampleUI.AddButton( IDC_FK_ZM, L"Rot Z-",100, iY, 60, 22 );

	iY += 24;

	g_SampleUI.AddButton( IDC_T_XP, L"Tra X+", 35, iY += 24, 60, 22 );
	g_SampleUI.AddButton( IDC_T_XM, L"Tra X-", 100, iY, 60, 22 );
	g_SampleUI.AddButton( IDC_T_YP, L"Tra Y+", 35, iY += 24, 60, 22 );
	g_SampleUI.AddButton( IDC_T_YM, L"Tra Y-", 100, iY, 60, 22 );
	g_SampleUI.AddButton( IDC_T_ZP, L"Tra Z+", 35, iY += 24, 60, 22 );
	g_SampleUI.AddButton( IDC_T_ZM, L"Tra Z-",100, iY, 60, 22 );

	iY += 24;

    swprintf_s( sz, 100, L"Motion Speed: %0.2f", g_dspeed );
    g_SampleUI.AddStatic( IDC_SPEED_STATIC, sz, 35, iY += 24, 125, 22 );
    g_SampleUI.AddSlider( IDC_SPEED, 50, iY += 24, 100, 22, 3, 30, ( int )( g_dspeed * 10.0f ) );


	s_timelineWnd = new OrgWindow(
							0,
							L"TimeLine",				//EBhENX
							GetModuleHandle(NULL),	//CX^Xnh
							WindowPos(0, 0),		//ʒu
							WindowSize(250,540),	//TCY
							L"TimeLine",				//^Cg
							s_mainwnd,					//eEBhEnh
							true,					//\E\
							70,50,70 );				//J[


	// EBhE̕{^̃CxgXi[
	// ItOcloseFlagIɂ郉_֐o^
	s_timelineWnd->setCloseListener( [](){ s_closeFlag=true; } );

	// EBhẼL[{[hCxgXi[
	// Rs[/Jbg/y[XgtOcopyFlag/cutFlag/pasteFlagIɂ郉_֐o^
	// Rs[̃L[{[hgp鏈̓L[{[hCxgXi[gpȂĂ
	// C[vŃ}Ct[L[ԂĎ邱Ƃō쐬\łB
	s_timelineWnd->setKeyboardEventListener( [](const KeyboardEvent &e){
		if( e.ctrlKey && !e.repeat && e.onDown ){
			switch( e.keyCode ){
			case 'C': 
				s_copyFlag=true; 
				break;
			case 'B':
				s_symcopyFlag=true;
				break;
			case 'X': 
				s_cutFlag=true; 
				break;
			case 'V': 
				s_pasteFlag=true; 
				break;
			case 'P': 
				s_previewFlag=1;
				break;
			case 'S': 
				s_previewFlag=0; 
				break;
			case 'D':
				s_deleteFlag=true;
				break;
			default:
				break;
			}
		}
	} );
/////////
	s_owpPlayerButton = new OWP_PlayerButton;
	s_timelineWnd->addParts(*s_owpPlayerButton);

	s_owpPlayerButton->setFrontPlayButtonListener( [](){ s_previewFlag = 1; } );
	s_owpPlayerButton->setBackPlayButtonListener( [](){  s_previewFlag = -1; } );
	s_owpPlayerButton->setFrontStepButtonListener( [](){ s_nextkeyFlag = true; } );
	s_owpPlayerButton->setBackStepButtonListener( [](){  s_befkeyFlag = true; } );
	s_owpPlayerButton->setStopButtonListener( [](){  s_previewFlag = 0; } );
	s_owpPlayerButton->setResetButtonListener( [](){ if( s_owpTimeline ){ s_owpTimeline->setCurrentTime( 0.0 ); } } );

/////////
	// c[EBhE쐬ă{^ނǉ
	s_toolWnd= new OrgWindow(
							1,
							_T("TimeLineTool"),		//EBhENX
							GetModuleHandle(NULL),	//CX^Xnh
							WindowPos(100, 100),		//ʒu
							WindowSize(150,10),		//TCY
							_T("c[EBhE"),	//^Cg
							s_timelineWnd->getHWnd(),	//eEBhEnh
							true,					//\E\
							70,50,70,				//J[
							true,					//邩ۂ
							false);					//TCYύX̉

	s_toolCopyB = new OWP_Button(_T("Rs["));;
	s_toolSymCopyB = new OWP_Button(_T("Ώ̃Rs["));;
	s_toolCutB = new OWP_Button(_T("Jbg"));;
	s_toolPasteB = new OWP_Button(_T("y[Xg"));;
	s_toolDeleteB = new OWP_Button(_T("폜"));;

	s_toolWnd->addParts(*s_toolCopyB);
	s_toolWnd->addParts(*s_toolSymCopyB);
	s_toolWnd->addParts(*s_toolCutB);
	s_toolWnd->addParts(*s_toolPasteB);
	s_toolWnd->addParts(*s_toolDeleteB);

	s_toolWnd->setCloseListener( [](){ s_closetoolFlag=true; } );
	s_toolCopyB->setButtonListener( [](){ s_copyFlag = true; } );
	s_toolSymCopyB->setButtonListener( [](){ s_symcopyFlag = true; } );
	s_toolCutB->setButtonListener( [](){ s_cutFlag = true; } );
	s_toolPasteB->setButtonListener( [](){ s_pasteFlag = true; } );
	s_toolDeleteB->setButtonListener( [](){ s_deleteFlag = true; } );

////
	// EBhE쐬
	s_layerWnd = new OrgWindow(
							1,
							_T("LayerTool"),		//EBhENX
							GetModuleHandle(NULL),	//CX^Xnh
							WindowPos(800, 0),		//ʒu
							WindowSize(150,200),		//TCY
							_T("IuWFNgpl"),	//^Cg
							NULL,					//eEBhEnh
							true,					//\E\
							70,50,70,				//J[
							true,					//邩ۂ
							true);					//TCYύX̉

	// C[EBhEp[c쐬
	s_owpLayerTable= new OWP_LayerTable(_T("C[e[u"));
	WCHAR label[256];
	wcscpy_s( label, 256, L"dummy name" );
	s_owpLayerTable->newLine( label, 0 );

	// EBhEɃEBhEp[co^
	s_layerWnd->addParts(*s_owpLayerTable);


	s_layerWnd->setCloseListener( [](){ s_closeobjFlag=true; } );

	//C[̃J[\Xi[
	s_owpLayerTable->setCursorListener( [](){
		//_tprintf_s( _T("CurrentLayer: Index=%3d Name=%s\n"),
		//			owpLayerTable->getCurrentLine(),
		//			owpLayerTable->getCurrentLineName().c_str() );
	} );

	//C[̈ړXi[
	s_owpLayerTable->setLineShiftListener( [](int from, int to){
		//_tprintf_s( _T("ShiftLayer: fromIndex=%3d toIndex=%3d\n"), from, to );
	} );

	//C[̉ԕύXXi[
	s_owpLayerTable->setChangeVisibleListener( [](int index){
		if( s_model ){
			CMQOObject* curobj = (CMQOObject*)( s_owpLayerTable->getObj(index) );
			if( curobj ){
				if( s_owpLayerTable->getVisible(index) ){
					curobj->m_dispflag = 1;				
				}else{
					curobj->m_dispflag = 0;
				}
			}
		}
	} );

	//C[̃bNԕύXXi[
	s_owpLayerTable->setChangeLockListener( [](int index){
		//if( owpLayerTable->getLock(index) ){
		//	_tprintf_s( _T("ChangeLock: Index=%3d Lock='True'  Name=%s\n"),
		//				index,
		//				owpLayerTable->getName(index).c_str() );
		//}else{
		//	_tprintf_s( _T("ChangeLock: Index=%3d Lock='False' Name=%s\n"),
		//				index,
		//				owpLayerTable->getName(index).c_str() );
		//}
	} );

	//C[̃vpeBR[Xi[
	s_owpLayerTable->setCallPropertyListener( [](int index){
		//_tprintf_s( _T("CallProperty: Index=%3d Name=%s\n"),
		//			index,
		//			owpLayerTable->getName(index).c_str() );
	} );

}


//--------------------------------------------------------------------------------------
// Called during device initialization, this code checks the device for some 
// minimum set of capabilities, and rejects those that don't pass by returning E_FAIL.
//------------------------------------------------
bool CALLBACK IsDeviceAcceptable( D3DCAPS9* pCaps, D3DFORMAT AdapterFormat,
                                  D3DFORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    // No fallback defined by this app, so reject any device that 
    // doesn't support at least ps2.0
    if( pCaps->PixelShaderVersion < D3DPS_VERSION( 3, 0 ) )
        return false;

	if( pCaps->MaxVertexIndex <= 0x0000FFFF ){
		return false;
	}

    // Skip backbuffer formats that don't support alpha blending
    IDirect3D9* pD3D = DXUTGetD3D9Object();
    if( FAILED( pD3D->CheckDeviceFormat( pCaps->AdapterOrdinal, pCaps->DeviceType,
                                         AdapterFormat, D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING,
                                         D3DRTYPE_TEXTURE, BackBufferFormat ) ) )
        return false;

    return true;
}


//--------------------------------------------------------------------------------------
// This callback function is called immediately before a device is created to allow the 
// application to modify the device settings. The supplied pDeviceSettings parameter 
// contains the settings that the framework has selected for the new device, and the 
// application can make any desired changes directly to this structure.  Note however that 
// DXUT will not correct invalid device settings so care must be taken 
// to return valid device settings, otherwise IDirect3D9::CreateDevice() will fail.  
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
    assert( DXUT_D3D9_DEVICE == pDeviceSettings->ver );

    HRESULT hr;
    IDirect3D9* pD3D = DXUTGetD3D9Object();
    D3DCAPS9 caps;

    V( pD3D->GetDeviceCaps( pDeviceSettings->d3d9.AdapterOrdinal,
                            pDeviceSettings->d3d9.DeviceType,
                            &caps ) );

    // If device doesn't support HW T&L or doesn't support 1.1 vertex shaders in HW 
    // then switch to SWVP.
    if( ( caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT ) == 0 ||
        caps.VertexShaderVersion < D3DVS_VERSION( 1, 1 ) )
    {
        pDeviceSettings->d3d9.BehaviorFlags = D3DCREATE_SOFTWARE_VERTEXPROCESSING;
    }

    // Debugging vertex shaders requires either REF or software vertex processing 
    // and debugging pixel shaders requires REF.  
#ifdef DEBUG_VS
    if( pDeviceSettings->d3d9.DeviceType != D3DDEVTYPE_REF )
    {
        pDeviceSettings->d3d9.BehaviorFlags &= ~D3DCREATE_HARDWARE_VERTEXPROCESSING;
        pDeviceSettings->d3d9.BehaviorFlags &= ~D3DCREATE_PUREDEVICE;
        pDeviceSettings->d3d9.BehaviorFlags |= D3DCREATE_SOFTWARE_VERTEXPROCESSING;
    }
#endif
#ifdef DEBUG_PS
    pDeviceSettings->d3d9.DeviceType = D3DDEVTYPE_REF;
#endif
    // For the first device created if its a REF device, optionally display a warning dialog box
    static bool s_bFirstTime = true;
    if( s_bFirstTime )
    {
        s_bFirstTime = false;
        if( pDeviceSettings->d3d9.DeviceType == D3DDEVTYPE_REF )
            DXUTDisplaySwitchingToREFWarning( pDeviceSettings->ver );
    }

    return true;
}


//--------------------------------------------------------------------------------------
// This callback function will be called immediately after the Direct3D device has been 
// created, which will happen during application initialization and windowed/full screen 
// toggles. This is the best location to create D3DPOOL_MANAGED resources since these 
// resources need to be reloaded whenever the device is destroyed. Resources created  
// here should be released in the OnDestroyDevice callback. 
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc,
                                 void* pUserContext )
{
    HRESULT hr;

	s_pdev = pd3dDevice;

    V_RETURN( g_DialogResourceManager.OnD3D9CreateDevice( pd3dDevice ) );
    V_RETURN( g_SettingsDlg.OnD3D9CreateDevice( pd3dDevice ) );
    // Initialize the font
    V_RETURN( D3DXCreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET,
                              OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE,
                              L"Arial", &g_pFont ) );

	s_totalmb.center = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	s_totalmb.max = D3DXVECTOR3( 5.0f, 5.0f, 5.0f );
	s_totalmb.min = D3DXVECTOR3( -5.0f, -5.0f, -5.0f );
	s_totalmb.r = D3DXVec3Length( &s_totalmb.max );
	g_vCenter = s_totalmb.center;
	float fObjectRadius = s_totalmb.r;
    D3DXMatrixTranslation( &g_mCenterWorld, -g_vCenter.x, -g_vCenter.y, -g_vCenter.z );

    V_RETURN( CDXUTDirectionWidget::StaticOnD3D9CreateDevice( pd3dDevice ) );
    for( int i = 0; i < MAX_LIGHTS; i++ )
		g_LightControl[i].SetRadius( fObjectRadius );
 
    // Define DEBUG_VS and/or DEBUG_PS to debug vertex and/or pixel shaders with the 
    // shader debugger. Debugging vertex shaders requires either REF or software vertex 
    // processing, and debugging pixel shaders requires REF.  The 
    // D3DXSHADER_FORCE_*_SOFTWARE_NOOPT flag improves the debug experience in the 
    // shader debugger.  It enables source level debugging, prevents instruction 
    // reordering, prevents dead code elimination, and forces the compiler to compile 
    // against the next higher available software target, which ensures that the 
    // unoptimized shaders do not exceed the shader model limitations.  Setting these 
    // flags will cause slower rendering since the shaders will be unoptimized and 
    // forced into software.  See the DirectX documentation for more information about 
    // using the shader debugger.
    DWORD dwShaderFlags = D3DXFX_NOT_CLONEABLE;

#if defined( DEBUG ) || defined( _DEBUG )
    // Set the D3DXSHADER_DEBUG flag to embed debug information in the shaders.
    // Setting this flag improves the shader debugging experience, but still allows 
    // the shaders to be optimized and to run exactly the way they will run in 
    // the release configuration of this program.
    dwShaderFlags |= D3DXSHADER_DEBUG;
    #endif

#ifdef DEBUG_VS
        dwShaderFlags |= D3DXSHADER_FORCE_VS_SOFTWARE_NOOPT;
    #endif
#ifdef DEBUG_PS
        dwShaderFlags |= D3DXSHADER_FORCE_PS_SOFTWARE_NOOPT;
    #endif

    // Preshaders are parts of the shader that the effect system pulls out of the 
    // shader and runs on the host CPU. They should be used if you are GPU limited. 
    // The D3DXSHADER_NO_PRESHADER flag disables preshaders.
    if( !g_bEnablePreshader )
        dwShaderFlags |= D3DXSHADER_NO_PRESHADER;

    // Read the D3DX effect file
    WCHAR str[MAX_PATH];
    V_RETURN( DXUTFindDXSDKMediaFileCch( str, MAX_PATH, L"Shader\\Ochakko.fx" ) );

    // If this fails, there should be debug output as to 
    // why the .fx file failed to compile
    V_RETURN( D3DXCreateEffectFromFile( pd3dDevice, str, NULL, NULL, dwShaderFlags, NULL, &g_pEffect, NULL ) );

	CallF( GetShaderHandle(), return 1 );

    // Setup the camera's view parameters
    D3DXVECTOR3 vecEye( 0.0f, 0.0f, -15.0f );
    D3DXVECTOR3 vecAt ( 0.0f, 0.0f, -0.0f );
    g_Camera.SetViewParams( &vecEye, &vecAt );
    g_Camera.SetRadius( fObjectRadius * 3.0f, fObjectRadius * 0.5f, fObjectRadius * 10.0f );


	s_select = new CModel();
	if( !s_select ){
		_ASSERT( 0 );
		return 1;
	}
	CallF( s_select->LoadMQO( s_pdev, L"RDBMedia\\select.mqo", 1 ), return 1 );
	CallF( s_select->CalcInf(), return 1 );
	CallF( s_select->MakeDispObj(), return 1 );
//	CallF( s_select->DbgDump(), return 1 );

	s_bmark = new CModel();
	if( !s_bmark ){
		_ASSERT( 0 );
		return 1;
	}
	CallF( s_bmark->LoadMQO( s_pdev, L"RDBMedia\\bonemark.mqo", 1 ), return 1 );
	CallF( s_bmark->CalcInf(), return 1 );
	CallF( s_bmark->MakeDispObj(), return 1 );

	s_bcircle = new CMySprite( s_pdev );
	if( !s_bcircle ){
		_ASSERT( 0 );
		return 1;
	}

	WCHAR path[MAX_PATH];
	wcscpy_s( path, MAX_PATH, g_basedir );
	WCHAR* lasten = 0;
	WCHAR* last2en = 0;
	lasten = wcsrchr( path, TEXT( '\\' ) );
	*lasten = 0L;
	last2en = wcsrchr( path, TEXT( '\\' ) );
	*last2en = 0L;
	wcscat_s( path, MAX_PATH, L"\\Media\\RDBMedia\\" );
	CallF( s_bcircle->Create( path, L"bonecircle.dds", 0, D3DPOOL_DEFAULT, 0 ), return 1 );


	s_pdev->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	s_pdev->SetRenderState( D3DRS_BLENDOP, D3DBLENDOP_ADD );
	s_pdev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
	s_pdev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
	s_pdev->SetRenderState( D3DRS_ALPHATESTENABLE, TRUE );
	s_pdev->SetRenderState( D3DRS_ALPHAREF, 0x00 );
	s_pdev->SetRenderState( D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL );


    return S_OK;
}


//--------------------------------------------------------------------------------------
// This callback function will be called immediately after the Direct3D device has been 
// reset, which will happen after a lost device scenario. This is the best location to 
// create D3DPOOL_DEFAULT resources since these resources need to be reloaded whenever 
// the device is lost. Resources created here should be released in the OnLostDevice 
// callback. 
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnResetDevice( IDirect3DDevice9* pd3dDevice,
                                const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
    HRESULT hr;

	if( g_texbank ){
		CallF( g_texbank->Invalidate(), return 1 );
		CallF( g_texbank->Restore(), return 1 );
	}

    V_RETURN( g_DialogResourceManager.OnD3D9ResetDevice() );
    V_RETURN( g_SettingsDlg.OnD3D9ResetDevice() );

    if( g_pFont )
        V_RETURN( g_pFont->OnResetDevice() );
    if( g_pEffect )
        V_RETURN( g_pEffect->OnResetDevice() );

    // Create a sprite to help batch calls when drawing many lines of text
    V_RETURN( D3DXCreateSprite( pd3dDevice, &g_pSprite ) );

    for( int i = 0; i < MAX_LIGHTS; i++ )
        g_LightControl[i].OnD3D9ResetDevice( pBackBufferSurfaceDesc );

    // Setup the camera's projection parameters
    float fAspectRatio = pBackBufferSurfaceDesc->Width / ( FLOAT )pBackBufferSurfaceDesc->Height;
    g_Camera.SetProjParams( D3DX_PI / 4, fAspectRatio, 10.0f, 40000.0f );
    g_Camera.SetWindow( pBackBufferSurfaceDesc->Width, pBackBufferSurfaceDesc->Height );
    g_Camera.SetButtonMasks( MOUSE_LEFT_BUTTON, MOUSE_WHEEL, MOUSE_MIDDLE_BUTTON );

    g_HUD.SetLocation( pBackBufferSurfaceDesc->Width - 170, 0 );
    g_HUD.SetSize( 170, 170 );
    g_SampleUI.SetLocation( pBackBufferSurfaceDesc->Width - 170, pBackBufferSurfaceDesc->Height - 550 );
    g_SampleUI.SetSize( 170, 400 );

    return S_OK;
}


//--------------------------------------------------------------------------------------
// This callback function will be called once at the beginning of every frame. This is the
// best location for your application to handle updates to the scene, but is not 
// intended to contain actual rendering calls, which should instead be placed in the 
// OnFrameRender callback.  
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
	static double savetime = 0.0;

    // Update the camera's position based on user input 
    g_Camera.FrameMove( fElapsedTime );

	double difftime = fTime - savetime;

	if( s_previewFlag && s_model && s_model->m_curmotinfo ){
		double nextframe = 0.0;
		if( s_previewFlag == 1 ){
			double curspeed, curframe;
			CallF( s_model->GetMotionSpeed( &curspeed ), return );
			CallF( s_model->GetMotionFrame( &curframe ), return );
			double oneframe = 1.0 / 60.0;
			nextframe = curframe + difftime / oneframe * curspeed;
			if( nextframe > s_model->m_curmotinfo->frameleng ){
				nextframe = curframe;
				s_previewFlag = 0;
			}
		}else if( s_previewFlag == -1 ){
			double curspeed, curframe;
			CallF( s_model->GetMotionSpeed( &curspeed ), return );
			CallF( s_model->GetMotionFrame( &curframe ), return );
			double oneframe = 1.0 / 60.0;
			nextframe = curframe - difftime / oneframe * curspeed;
			if( nextframe < 0.0 ){
				nextframe = curframe;
				s_previewFlag = 0;
			}
		}

		vector<MODELELEM>::iterator itrmodel;
		for( itrmodel = s_modelindex.begin(); itrmodel != s_modelindex.end(); itrmodel++ ){
			CModel* curmodel = itrmodel->modelptr;
			if( curmodel && curmodel->m_curmotinfo ){
				curmodel->SetMotionFrame( nextframe );
			}
		}
		s_owpTimeline->setCurrentTime( nextframe, true );
	}
	savetime = fTime;


	// ItOmF //////////////////////////////////////////////////////////
	if( s_closeFlag ){
		s_closeFlag= false;
		if( s_timelineWnd ){
			s_timelineWnd->setVisible( 0 );
		}
		s_dispmw = false;
	}
	if( s_closetoolFlag ){
		s_closetoolFlag= false;
		if( s_toolWnd ){
			s_toolWnd->setVisible( 0 );
		}
		s_disptool = false;
	}
	if( s_closeobjFlag ){
		s_closeobjFlag= false;
		if( s_layerWnd ){
			s_layerWnd->setVisible( 0 );
		}
		s_dispobj = false;
	}
	

	// J[\ړtOmF //////////////////////////////////////////////////
	if( s_cursorFlag ){
		s_cursorFlag= false;

		// J[\ʒuR\[ɏo
		int curlineno = s_owpTimeline->getCurrentLine();// Is
		if( curlineno >= 0 ){
			s_curboneno = s_lineno2boneno[ curlineno ];

			if( s_model ){
				CDXUTComboBox* pComboBox;
				pComboBox = g_SampleUI.GetComboBox( IDC_COMBO_BONE );
				CBone* pBone;
				pBone = s_model->m_bonelist[ s_curboneno ];
				if( pBone ){
				    pComboBox->SetSelectedByData( ULongToPtr( s_curboneno ) );
				}
			}
		}else{
			s_curboneno = -1;
		}
		double curframe = s_owpTimeline->getCurrentTime();// I
		vector<MODELELEM>::iterator itrmodel;
		for( itrmodel = s_modelindex.begin(); itrmodel != s_modelindex.end(); itrmodel++ ){
			CModel* curmodel = itrmodel->modelptr;
			if( curmodel && curmodel->m_curmotinfo ){
				curmodel->SetMotionFrame( curframe );
			}
		}

		//DbgOut( L"cursor : lineno %d, boneno %d, frame %f\r\n", curlineno, s_curboneno, s_curframe );
	}


	// Rs[/Jbg/y[XgtOmF ////////////////////////////////////////
	if( s_copyFlag && s_model ){
		s_copyFlag= false;
		s_copyKeyInfoList.erase( s_copyKeyInfoList.begin(), s_copyKeyInfoList.end() );
		s_copyKeyInfoList = s_owpTimeline->getSelectedKey();

		list<OWP_Timeline::KeyInfo>::iterator itrcp;
		for( itrcp = s_copyKeyInfoList.begin(); itrcp != s_copyKeyInfoList.end(); itrcp++ ){
			CMotionPoint* mpptr = (CMotionPoint*)(itrcp->object);
			itrcp->eul = mpptr->m_eul;
			itrcp->tra = mpptr->m_tra;
		}
	}
	if( s_symcopyFlag && s_model ){
		s_symcopyFlag = false;

		s_copyKeyInfoList.erase( s_copyKeyInfoList.begin(), s_copyKeyInfoList.end() );
		s_copyKeyInfoList = s_owpTimeline->getSelectedKey();

		list<OWP_Timeline::KeyInfo>::iterator itrcp;
		for( itrcp = s_copyKeyInfoList.begin(); itrcp != s_copyKeyInfoList.end(); itrcp++ ){
			int srcboneno = s_lineno2boneno[ itrcp->lineIndex ];
			if( srcboneno >= 0 ){
				int symboneno = -1;
				int existflag = 0;
				CallF( s_model->GetSymBoneNo( srcboneno, &symboneno, &existflag ), return );
				if( existflag ){
					CMotionPoint symmp;
					int existmp = 0;
					CallF( s_model->m_bonelist[ symboneno ]->CalcMotionPoint( s_model->m_curmotinfo->motid, itrcp->time, &symmp, &existmp ), return );
					CQuaternion symq;
					symq.SetParams( 1.0f, 0.0f, 0.0f, 0.0f );
					symmp.m_q.CalcSym( &symq );
					CMotionPoint* pbef = 0;
					CMotionPoint* pnext = 0;
					int existcur = 0;
					CallF( s_model->m_bonelist[ symboneno ]->GetBefNextMP( s_model->m_curmotinfo->motid, itrcp->time, &pbef, &pnext, &existcur ), return );
					D3DXVECTOR3 cpeul;
					D3DXVECTOR3 befeul;
					if( pbef ){
						befeul = pbef->m_eul;
					}else{
						befeul = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
					}
					CallF( symq.Q2Eul( befeul, &cpeul ), return );
					itrcp->eul = cpeul;
					itrcp->tra.x = -symmp.m_tra.x;
					itrcp->tra.y = symmp.m_tra.y;
					itrcp->tra.z = symmp.m_tra.z;
				}else{
					CMotionPoint* mpptr = (CMotionPoint*)(itrcp->object);
					itrcp->eul = mpptr->m_eul;
					itrcp->tra = mpptr->m_tra;
				}

			}
		}
	}


	if( s_cutFlag && s_model ){
		s_cutFlag= false;
		s_copyKeyInfoList.erase( s_copyKeyInfoList.begin(), s_copyKeyInfoList.end() );
		s_copyKeyInfoList = s_owpTimeline->getSelectedKey();

		list<OWP_Timeline::KeyInfo>::iterator itrcp;
		for( itrcp = s_copyKeyInfoList.begin(); itrcp != s_copyKeyInfoList.end(); itrcp++ ){
			CMotionPoint* mpptr = (CMotionPoint*)(itrcp->object);
			itrcp->eul = mpptr->m_eul;
			itrcp->tra = mpptr->m_tra;

			//motionpointdeletedelete ListenerłB
		}
		s_owpTimeline->deleteKey();
	}
	if( s_deleteFlag && s_model ){
		s_deleteFlag = false;
		s_owpTimeline->deleteKey();
		//motionpointdeletedelete ListenerłB
	}

	if( s_pasteFlag && s_model ){
		s_pasteFlag= false;
		//Rs[ꂽL[̐擪߂
		double copyStartTime= DBL_MAX;
		for(list<OWP_Timeline::KeyInfo>::iterator itr=s_copyKeyInfoList.begin();
			itr!=s_copyKeyInfoList.end(); itr++){
			if( itr->time<=copyStartTime ){
				copyStartTime= itr->time;
			}
		}

		//y[Xg
		s_owpTimeline->setRewriteOnChangeFlag(false);		//SẴL[y[XgI܂ōĕ`vXgbv
		for(list<OWP_Timeline::KeyInfo>::iterator itr=s_copyKeyInfoList.begin(); itr!=s_copyKeyInfoList.end(); itr++){
			CMotionPoint* newmp = 0;
			int existflag = 0;
			_ASSERT( s_model->m_curmotinfo );
			newmp = s_model->AddMotionPoint( s_model->m_curmotinfo->motid, itr->time - copyStartTime + s_owpTimeline->getCurrentTime(),
				s_lineno2boneno[ itr->lineIndex ],
				1,
				&existflag );
			newmp->SetEul( itr->eul );
			newmp->m_tra = itr->tra;

			if( existflag == 0 ){
				int boneno = s_lineno2boneno[ itr->lineIndex ];
				CBone* boneptr = s_model->m_bonelist[ boneno ];
				if( boneptr ){
					s_owpTimeline->newKey(
						//itr->label,//<--- Ԃ폜Ă\B
						boneptr->m_wbonename,
						itr->time - copyStartTime + s_owpTimeline->getCurrentTime(),
						(void*)newmp);
				}
			}
		}
		s_owpTimeline->callRewrite();						//ĕ`
		s_owpTimeline->setRewriteOnChangeFlag(true);		//ĕ`vĊJ
	}

	// L[ItOmF ///////////////////////////////////////////////////////
	if( s_selectFlag ){
		s_selectFlag= false;
		s_selectKeyInfoList.erase( s_selectKeyInfoList.begin(), s_selectKeyInfoList.end() );
		s_selectKeyInfoList = s_owpTimeline->getSelectedKey();
	}

	// L[ړtOmF ///////////////////////////////////////////////////////////
	if( s_keyShiftFlag ){
		s_keyShiftFlag= false;

		if( s_keyShiftTime != 0.0 ){

			// ړʂR\[ɏo
			//printf_s("ShiftTime:%5f\n",s_keyShiftTime);

			list<OWP_Timeline::KeyInfo>::iterator itrsel;
			for( itrsel = s_selectKeyInfoList.begin(); itrsel != s_selectKeyInfoList.end(); itrsel++ ){
				CMotionPoint* mpptr = (CMotionPoint*)(itrsel->object);

				CBone* boneptr = s_model->m_bonelist[ s_lineno2boneno[ itrsel->lineIndex ] ];
				boneptr->OrderMotionPoint( s_model->m_curmotinfo->motid, itrsel->time + s_keyShiftTime, mpptr, 0 );
			}


			// IĂL[SĈړ
			// ݂̏̂s_֐ftHgŐݒ肳Ă
			s_owpTimeline->shiftKeyTime( s_keyShiftTime );

		}
		s_selectKeyInfoList.erase( s_selectKeyInfoList.begin(), s_selectKeyInfoList.end() );
		s_selectKeyInfoList = s_owpTimeline->getSelectedKey();

	}

	if( s_owpTimeline && s_befkeyFlag && s_model && s_model->m_curmotinfo ){
		s_befkeyFlag = false;
		double curframe = s_owpTimeline->getCurrentTime();
		int curline = s_owpTimeline->getCurrentLine();
		int curboneno = s_lineno2boneno[ curline ];
		CBone* curbone = s_model->m_bonelist[ curboneno ];
		if( curbone ){
			CMotionPoint* pbef = 0;
			CMotionPoint* pnext = 0;
			int existflag = 0;
			curbone->GetBefNextMP( s_model->m_curmotinfo->motid, curframe, &pbef, &pnext, &existflag );
			if( existflag ){
				pbef = pbef->m_prev;
			}
			if( pbef ){
				s_owpTimeline->setCurrentTime( pbef->m_frame );
			}else{
				s_owpTimeline->setCurrentTime( 0.0 );
			}
		}
	}
	if( s_owpTimeline && s_nextkeyFlag && s_model && s_model->m_curmotinfo ){
		s_nextkeyFlag = false;
		double curframe = s_owpTimeline->getCurrentTime();
		int curline = s_owpTimeline->getCurrentLine();
		int curboneno = s_lineno2boneno[ curline ];
		CBone* curbone = s_model->m_bonelist[ curboneno ];
		if( curbone ){
			CMotionPoint* pbef = 0;
			CMotionPoint* pnext = 0;
			int existflag = 0;
			curbone->GetBefNextMP( s_model->m_curmotinfo->motid, curframe, &pbef, &pnext, &existflag );
			if( pnext ){
				s_owpTimeline->setCurrentTime( pnext->m_frame );
			}else{
				s_owpTimeline->setCurrentTime( s_model->m_curmotinfo->frameleng );
			}
		}
	}

	// 폜ꂽL[̃X^bNmF ////////////////////////////////////////////
	for(; s_deletedKeyInfoList.begin()!=s_deletedKeyInfoList.end();
		s_deletedKeyInfoList.pop_front()){

		CMotionPoint* mpptr = (CMotionPoint*)( s_deletedKeyInfoList.begin()->object );
		CBone* boneptr = s_model->m_bonelist[ s_lineno2boneno[ s_deletedKeyInfoList.begin()->lineIndex ] ];
		mpptr->LeaveFromChain( s_model->m_curmotinfo->motid, boneptr );
		delete mpptr;
	}
}


//--------------------------------------------------------------------------------------
// This callback function will be called at the end of every frame to perform all the 
// rendering calls for the scene, and it will also be called if the window needs to be 
// repainted. After this function has returned, DXUT will call 
// IDirect3DDevice9::Present to display the contents of the next buffer in the swap chain
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameRender( IDirect3DDevice9* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
	GetKeyboardState( (PBYTE)g_keybuf );


    // If the settings dialog is being shown, then
    // render it instead of rendering the app's scene
    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.OnRender( fElapsedTime );
        return;
    }

    HRESULT hr;
    D3DXVECTOR3 vLightDir[MAX_LIGHTS];
    D3DXCOLOR vLightDiffuse[MAX_LIGHTS];
    D3DXMATRIXA16 mView;
    D3DXMATRIXA16 mProj;

    // Clear the render target and the zbuffer 
    V( pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DXCOLOR( 0.0f, 0.25f, 0.25f, 0.55f ), 1.0f,
                          0 ) );

	s_pdev->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
	s_pdev->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
	s_pdev->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );

    // Render the scene
    if( SUCCEEDED( pd3dDevice->BeginScene() ) )
    {
        // Get the projection & view matrix from the camera class
        mProj = *g_Camera.GetProjMatrix();
        mView = *g_Camera.GetViewMatrix();

        // Render the light arrow so the user can visually see the light dir
        for( int i = 0; i < g_nNumActiveLights; i++ )
        {
            D3DXCOLOR arrowColor = ( i == g_nActiveLight ) ? D3DXCOLOR( 1, 1, 0, 1 ) : D3DXCOLOR( 1, 1, 1, 1 );
            V( g_LightControl[i].OnRender9( arrowColor, &mView, &mProj, g_Camera.GetEyePt() ) );
            vLightDir[i] = g_LightControl[i].GetLightDirection();
            vLightDiffuse[i] = g_fLightScale * D3DXCOLOR( 1, 1, 1, 1 );
        }

		D3DXVECTOR3 lightamb( 1.0f, 1.0f, 1.0f );

        V( g_pEffect->SetValue( g_hLightDir, vLightDir, sizeof( D3DXVECTOR3 ) * MAX_LIGHTS ) );
        V( g_pEffect->SetValue( g_hLightDiffuse, vLightDiffuse, sizeof( D3DXVECTOR4 ) * MAX_LIGHTS ) );
		V( g_pEffect->SetValue( g_hLightAmbient, &lightamb, sizeof( D3DXVECTOR3 ) ) );
        V( g_pEffect->SetInt( g_hnNumLight, g_nNumActiveLights ) );
		D3DXVECTOR3 eyept = *g_Camera.GetEyePt();
		V( g_pEffect->SetValue( g_hEyePos, &eyept, sizeof( D3DXVECTOR3 ) ) );

		D3DXMATRIXA16 mtempW, mW, mVP;
		mW = g_mCenterWorld * *g_Camera.GetWorldMatrix();
		mVP = mView * mProj;

		V( g_pEffect->SetMatrix( g_hmVP, &mVP ) );
		V( g_pEffect->SetMatrix( g_hmWorld, &mW ) );

		vector<MODELELEM>::iterator itrmodel;
		for( itrmodel = s_modelindex.begin(); itrmodel != s_modelindex.end(); itrmodel++ ){
			CModel* curmodel = itrmodel->modelptr;

			if( curmodel ){
				curmodel->UpdateMatrix( &mW, &mVP );
				curmodel->SetShaderConst();
				int lightflag = 1;
				D3DXVECTOR4 diffusemult = D3DXVECTOR4(1.0f, 1.0f, 1.0f, 1.0f);
				_ASSERT( s_bmark && s_bcircle );
				curmodel->OnRender( s_pdev, lightflag, diffusemult );
			}
		}

		if( s_model ){
			s_model->RenderBoneMark( s_pdev, s_bmark, s_bcircle );
		}

		g_HUD.OnRender( fElapsedTime );
        g_SampleUI.OnRender( fElapsedTime );

        RenderText( fTime );

        V( pd3dDevice->EndScene() );
    }
}

//--------------------------------------------------------------------------------------
// Render the help and statistics text. This function uses the ID3DXFont interface for 
// efficient text rendering.
//--------------------------------------------------------------------------------------
void RenderText( double fTime )
{
	static double s_savetime = 0.0;

	double calcfps = 1.0 / (fTime - s_savetime);

    // The helper object simply helps keep track of text position, and color
    // and then it calls pFont->DrawText( m_pSprite, strMsg, -1, &rc, DT_NOCLIP, m_clr );
    // If NULL is passed in as the sprite object, then it will work fine however the 
    // pFont->DrawText() will not be batched together.  Batching calls will improves perf.
    CDXUTTextHelper txtHelper( g_pFont, g_pSprite, 15 );

    // Output statistics
    txtHelper.Begin();
    txtHelper.SetInsertionPos( 2, 0 );
    txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
    txtHelper.DrawTextLine( DXUTGetFrameStats( DXUTIsVsyncEnabled() ) );
    txtHelper.DrawTextLine( DXUTGetDeviceStats() );

    txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
    txtHelper.DrawFormattedTextLine( L"fps : %0.2f fTime: %0.1f  sin(fTime): %0.4f", calcfps, fTime, sin( fTime ) );

    // Draw help
    if( g_bShowHelp )
    {
        const D3DSURFACE_DESC* pd3dsdBackBuffer = DXUTGetD3D9BackBufferSurfaceDesc();
        txtHelper.SetInsertionPos( 2, pd3dsdBackBuffer->Height - 15 * 6 );
        txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 0.75f, 0.0f, 1.0f ) );
        txtHelper.DrawTextLine( L"Controls:" );

        txtHelper.SetInsertionPos( 20, pd3dsdBackBuffer->Height - 15 * 5 );
        txtHelper.DrawTextLine( L"Rotate model: Left mouse button\n"
                                L"Rotate light: Right mouse button\n"
                                L"Rotate camera: Middle mouse button\n"
                                L"Zoom camera: Mouse wheel scroll\n" );

        txtHelper.SetInsertionPos( 250, pd3dsdBackBuffer->Height - 15 * 5 );
        txtHelper.DrawTextLine( L"Hide help: F1\n"
                                L"Quit: ESC\n" );
    }
    else
    {
        txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
        txtHelper.DrawTextLine( L"Press F1 for help" );
    }
    txtHelper.End();

	s_savetime = fTime;
}


//--------------------------------------------------------------------------------------
// Before handling window messages, DXUT passes incoming windows 
// messages to the application through this callback function. If the application sets 
// *pbNoFurtherProcessing to TRUE, then DXUT will not process this message.
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, bool* pbNoFurtherProcessing,
                          void* pUserContext )
{

	if( uMsg == WM_COMMAND ){

		WORD menuid;
		menuid = LOWORD( wParam );

		if( (menuid >= 59900) && (menuid <= (59900 + MAXMOTIONNUM)) ){
			int selindex = menuid - 59900;
			OnAnimMenu( selindex );
			return 0;
		}else if( (menuid >= 61000) && (menuid <= (61000 + MAXMODELNUM)) ){
			int selindex = menuid - 61000;
			OnModelMenu( selindex );
			return 0;
		}else{
			switch( menuid ){
			case ID_FILE_OPEN40001:
				OpenMQOFile();
				return 0;
				break;
			case ID_DISPMW40002:
				DispMotionWindow();
				return 0;
				break;
			case 40007:
				DispToolWindow();
				return 0;
				break;
			case 40012:
				DispObjPanel();
				return 0;
				break;
			case ID_NEWMOT:
				AddMotion();
				return 0;
				break;
			case ID_DELCURMOT:
				OnDelMotion( s_curmotmenuindex );
				return 0;
				break;
			case ID_DELMODEL:
				OnDelModel( s_curmodelmenuindex );
				return 0;
				break;
			case ID_DELALLMODEL:
				OnDelAllModel();
				return 0;
				break;
			default:
				break;
			}
		}
	}

    // Always allow dialog resource manager calls to handle global messages
    // so GUI state is updated correctly
    *pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    if( g_SettingsDlg.IsActive() )
    {
        g_SettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
        return 0;
    }

    // Give the dialogs a chance to handle the message first
    *pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;
    *pbNoFurtherProcessing = g_SampleUI.MsgProc( hWnd, uMsg, wParam, lParam );
    if( *pbNoFurtherProcessing )
        return 0;

    g_LightControl[g_nActiveLight].HandleMessages( hWnd, uMsg, wParam, lParam );

    // Pass all remaining windows messages to camera so it can respond to user input
    g_Camera.HandleMessages( hWnd, uMsg, wParam, lParam );

    return 0;
}


//--------------------------------------------------------------------------------------
// As a convenience, DXUT inspects the incoming windows messages for
// keystroke messages and decodes the message parameters to pass relevant keyboard
// messages to the application.  The framework does not remove the underlying keystroke 
// messages, which are still passed to the application's MsgProc callback.
//--------------------------------------------------------------------------------------
void CALLBACK KeyboardProc( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
    if( bKeyDown )
    {
        switch( nChar )
        {
            case VK_F1:
                g_bShowHelp = !g_bShowHelp; break;
        }
    }
}


//--------------------------------------------------------------------------------------
// Handles the GUI events
//--------------------------------------------------------------------------------------
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{
    CDXUTComboBox* pComboBox;
	D3DXVECTOR3 cureul, neweul;
	int tmpboneno;
	CBone* tmpbone;
	WCHAR sz[100];
 
	float trastep = 10.0f;

    switch( nControlID )
    {
        case IDC_TOGGLEFULLSCREEN:
            DXUTToggleFullScreen(); break;
        case IDC_TOGGLEREF:
            DXUTToggleREF(); break;
        case IDC_CHANGEDEVICE:
            g_SettingsDlg.SetActive( !g_SettingsDlg.IsActive() ); break;

        case IDC_ENABLE_PRESHADER:
        {
            g_bEnablePreshader = g_SampleUI.GetCheckBox( IDC_ENABLE_PRESHADER )->GetChecked();

            if( DXUTGetD3D9Device() != NULL )
            {
                OnLostDevice( NULL );
                OnDestroyDevice( NULL );
                OnCreateDevice( DXUTGetD3D9Device(), DXUTGetD3D9BackBufferSurfaceDesc(), NULL );
                OnResetDevice( DXUTGetD3D9Device(), DXUTGetD3D9BackBufferSurfaceDesc(), NULL );
            }
            break;
        }

        case IDC_ACTIVE_LIGHT:
            if( !g_LightControl[g_nActiveLight].IsBeingDragged() )
            {
                g_nActiveLight++;
                g_nActiveLight %= g_nNumActiveLights;
            }
            break;

        case IDC_NUM_LIGHTS:
            if( !g_LightControl[g_nActiveLight].IsBeingDragged() )
            {
                WCHAR sz[100];
                swprintf_s( sz, 100, L"# Lights: %d", g_SampleUI.GetSlider( IDC_NUM_LIGHTS )->GetValue() );
                g_SampleUI.GetStatic( IDC_NUM_LIGHTS_STATIC )->SetText( sz );

                g_nNumActiveLights = g_SampleUI.GetSlider( IDC_NUM_LIGHTS )->GetValue();
                g_nActiveLight %= g_nNumActiveLights;
            }
            break;

        case IDC_LIGHT_SCALE:
            g_fLightScale = ( float )( g_SampleUI.GetSlider( IDC_LIGHT_SCALE )->GetValue() * 0.10f );

            swprintf_s( sz, 100, L"Light scale: %0.2f", g_fLightScale );
            g_SampleUI.GetStatic( IDC_LIGHT_SCALE_STATIC )->SetText( sz );
            break;

		
        case IDC_SPEED:
            g_dspeed = ( float )( g_SampleUI.GetSlider( IDC_SPEED )->GetValue() * 0.10f );
			if( s_model && s_model->m_curmotinfo ){
				s_model->SetMotionSpeed( g_dspeed );
			}

            swprintf_s( sz, 100, L"Motion Speed: %0.2f", g_dspeed );
            g_SampleUI.GetStatic( IDC_SPEED_STATIC )->SetText( sz );
            break;

		case IDC_COMBO_BONE:
			if( s_model ){
				pComboBox = g_SampleUI.GetComboBox( IDC_COMBO_BONE );
				tmpboneno = (int)PtrToUlong( pComboBox->GetSelectedData() );
				tmpbone = s_model->m_bonelist[ tmpboneno ];
				if( tmpbone ){
					s_curboneno = tmpboneno;
				}

				if( s_curboneno >= 0 ){
					int lineno = s_boneno2lineno[ s_curboneno ];
					if( lineno >= 0 ){
						s_owpTimeline->setCurrentLine( lineno, true );					
					}
				}
			}
			break;

		case IDC_FK_XP:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneEul( 10.0f, 0.0f, 0.0f );
			}
			break;
		case IDC_FK_XM:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneEul( -10.0f, 0.0f, 0.0f );
			}
			break;
		case IDC_FK_YP:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneEul( 0.0f, 10.0f, 0.0f );
			}
			break;
		case IDC_FK_YM:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneEul( 0.0f, -10.0f, 0.0f );
			}
			break;
		case IDC_FK_ZP:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneEul( 0.0f, 0.0f, 10.0f );
			}
			break;
		case IDC_FK_ZM:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneEul( 0.0f, 0.0f, -10.0f );
			}
			break;
		case IDC_T_XP:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneTra( trastep, 0.0f, 0.0f );
			}
			break;
		case IDC_T_XM:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneTra( -trastep, 0.0f, 0.0f );
			}
			break;
		case IDC_T_YP:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneTra( 0.0f, trastep, 0.0f );
			}
			break;
		case IDC_T_YM:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneTra( 0.0, -trastep, 0.0f );
			}
			break;
		case IDC_T_ZP:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneTra( 0.0f, 0.0f, trastep );
			}
			break;
		case IDC_T_ZM:
			if( s_model && (s_curboneno >= 0) ){
				AddBoneTra( 0.0f, 0.0f, -trastep );
			}
			break;
		default:
			break;
	
	}

}


//--------------------------------------------------------------------------------------
// This callback function will be called immediately after the Direct3D device has 
// entered a lost state and before IDirect3DDevice9::Reset is called. Resources created
// in the OnResetDevice callback should be released here, which generally includes all 
// D3DPOOL_DEFAULT resources. See the "Lost Devices" section of the documentation for 
// information about lost devices.
//--------------------------------------------------------------------------------------
void CALLBACK OnLostDevice( void* pUserContext )
{
    g_DialogResourceManager.OnD3D9LostDevice();
    g_SettingsDlg.OnD3D9LostDevice();
    CDXUTDirectionWidget::StaticOnD3D9LostDevice();
    if( g_pFont )
        g_pFont->OnLostDevice();
    if( g_pEffect )
        g_pEffect->OnLostDevice();
    SAFE_RELEASE( g_pSprite );

	if( g_texbank ){
		CallF( g_texbank->Invalidate(), return );
	}

}


//--------------------------------------------------------------------------------------
// This callback function will be called immediately after the Direct3D device has 
// been destroyed, which generally happens as a result of application termination or 
// windowed/full screen toggles. Resources created in the OnCreateDevice callback 
// should be released here, which generally includes all D3DPOOL_MANAGED resources. 
//--------------------------------------------------------------------------------------
void CALLBACK OnDestroyDevice( void* pUserContext )
{
    g_DialogResourceManager.OnD3D9DestroyDevice();
    g_SettingsDlg.OnD3D9DestroyDevice();
    CDXUTDirectionWidget::StaticOnD3D9DestroyDevice();
    SAFE_RELEASE( g_pEffect );
    SAFE_RELEASE( g_pFont );

	vector<MODELELEM>::iterator itrmodel;
	for( itrmodel = s_modelindex.begin(); itrmodel != s_modelindex.end(); itrmodel++ ){
		CModel* curmodel = itrmodel->modelptr;
		if( curmodel ){
			delete curmodel;
		}
	}
	s_modelindex.erase( s_modelindex.begin(), s_modelindex.end() );
	s_model = 0;

	if( s_select ){
		delete s_select;
		s_select = 0;
	}
	if( s_bmark ){
		delete s_bmark;
		s_bmark = 0;
	}
	if( s_bcircle ){
		delete s_bcircle;
		s_bcircle = 0;
	}


	if( g_texbank ){
		delete g_texbank;
		g_texbank = 0;
	}

	if( s_mainmenu ){
		DestroyMenu( s_mainmenu );
		s_mainmenu = 0;
	}

	DestroyTimeLine( 1 );

	if( s_timelineWnd ){
		delete s_timelineWnd;
		s_timelineWnd = 0;
	}

	if( s_toolWnd ){
		delete s_toolWnd;
		s_toolWnd = 0;
	}

	if( s_toolCopyB ){
		delete s_toolCopyB;
		s_toolCopyB = 0;
	}
	if( s_toolSymCopyB ){
		delete s_toolSymCopyB;
		s_toolSymCopyB = 0;
	}
	if( s_toolCutB ){
		delete s_toolCutB;
		s_toolCutB = 0;
	}
	if( s_toolPasteB ){
		delete s_toolPasteB;
		s_toolPasteB = 0;
	}
	if( s_toolDeleteB ){
		delete s_toolDeleteB;
		s_toolDeleteB = 0;
	}

	if( s_owpTimeline ){
		delete s_owpTimeline;
		s_owpTimeline = 0;
	}
	if( s_owpPlayerButton ){
		delete s_owpPlayerButton;
		s_owpPlayerButton = 0;
	}

	if( s_layerWnd ){
		delete s_layerWnd;
		s_layerWnd = 0;
	}
	if( s_owpLayerTable ){
		delete s_owpLayerTable;
		s_owpLayerTable = 0;
	}

}

int DestroyTimeLine( int dellist )
{
	if( dellist ){
		EraseKeyList();
	}

	s_tlarray.erase( s_tlarray.begin(), s_tlarray.end() );
	
	return 0;
}

int GetShaderHandle()
{
	if( !g_pEffect ){
		_ASSERT( 0 );
		return 1;
	}

	g_hRenderBoneL0 = g_pEffect->GetTechniqueByName( "RenderBoneL0" );
	_ASSERT( g_hRenderBoneL0 );
	g_hRenderBoneL1 = g_pEffect->GetTechniqueByName( "RenderBoneL1" );
	_ASSERT( g_hRenderBoneL1 );
	g_hRenderBoneL2 = g_pEffect->GetTechniqueByName( "RenderBoneL2" );
	_ASSERT( g_hRenderBoneL2 );
	g_hRenderBoneL3 = g_pEffect->GetTechniqueByName( "RenderBoneL3" );
	_ASSERT( g_hRenderBoneL3 );

	g_hRenderNoBoneL0 = g_pEffect->GetTechniqueByName( "RenderNoBoneL0" );
	_ASSERT( g_hRenderNoBoneL0 );
	g_hRenderNoBoneL1 = g_pEffect->GetTechniqueByName( "RenderNoBoneL1" );
	_ASSERT( g_hRenderNoBoneL1 );
	g_hRenderNoBoneL2 = g_pEffect->GetTechniqueByName( "RenderNoBoneL2" );
	_ASSERT( g_hRenderNoBoneL2 );
	g_hRenderNoBoneL3 = g_pEffect->GetTechniqueByName( "RenderNoBoneL3" );
	_ASSERT( g_hRenderNoBoneL3 );

	g_hRenderLine = g_pEffect->GetTechniqueByName( "RenderLine" );
	_ASSERT( g_hRenderLine );
	g_hRenderSprite = g_pEffect->GetTechniqueByName( "RenderSprite" );
	_ASSERT( g_hRenderSprite );


	g_hmWMatArray = g_pEffect->GetParameterByName( NULL, "g_mWMatArray" );
	_ASSERT( g_hmWMatArray );
	g_hmWorld = g_pEffect->GetParameterByName( NULL, "g_mWorld" );
	_ASSERT( g_hmWorld );
	g_hmVP = g_pEffect->GetParameterByName( NULL, "g_mVP" );
	_ASSERT( g_hmVP );
	g_hEyePos = g_pEffect->GetParameterByName( NULL, "g_EyePos" );
	_ASSERT( g_hEyePos );


	g_hnNumLight = g_pEffect->GetParameterByName( NULL, "g_nNumLights" );
	_ASSERT( g_hnNumLight );
	g_hLightDir = g_pEffect->GetParameterByName( NULL, "g_LightDir" );
	_ASSERT( g_hLightDir );
	g_hLightDiffuse = g_pEffect->GetParameterByName( NULL, "g_LightDiffuse" );
	_ASSERT( g_hLightDiffuse );
	g_hLightAmbient = g_pEffect->GetParameterByName( NULL, "g_LightAmbient" );
	_ASSERT( g_hLightAmbient );

	g_hdiffuse = g_pEffect->GetParameterByName( NULL, "g_diffuse" );
	_ASSERT( g_hdiffuse );
	g_hambient = g_pEffect->GetParameterByName( NULL, "g_ambient" );
	_ASSERT( g_hambient );
	g_hspecular = g_pEffect->GetParameterByName( NULL, "g_specular" );
	_ASSERT( g_hspecular );
	g_hpower = g_pEffect->GetParameterByName( NULL, "g_power" );
	_ASSERT( g_hpower );
	g_hemissive = g_pEffect->GetParameterByName( NULL, "g_emissive" );
	_ASSERT( g_hemissive );
	g_hMeshTexture = g_pEffect->GetParameterByName( NULL, "g_MeshTexture" );
	_ASSERT( g_hMeshTexture );

	return 0;
}

int SetBaseDir()
{
	WCHAR filename[MAX_PATH] = {0};
	WCHAR* endptr = 0;

	int ret;
	ret = GetModuleFileNameW( NULL, filename, MAX_PATH );
	if( ret == 0 ){
		_ASSERT( 0 );
		return 1;
	}


    WCHAR* lasten = NULL;
    lasten = wcsrchr( filename, TEXT( '\\' ) );
	if( !lasten ){
		_ASSERT( 0 );
		DbgOut( L"SetMediaDir : strrchr error !!!\n" );
		return 1;
	}

	*lasten = 0;

	WCHAR* last2en = 0;
	WCHAR* last3en = 0;
	last2en = wcsrchr( filename, TEXT( '\\' ) );
	if( last2en ){
		*last2en = 0;
		last3en = wcsrchr( filename, TEXT( '\\' ) );
		if( last3en ){
			if( (wcscmp( last2en + 1, L"debug" ) == 0) ||
				(wcscmp( last2en + 1, L"Debug" ) == 0) ||
				(wcscmp( last2en + 1, L"DEBUG" ) == 0) ||
				(wcscmp( last2en + 1, L"release" ) == 0) ||
				(wcscmp( last2en + 1, L"Release" ) == 0) ||
				(wcscmp( last2en + 1, L"RELEASE" ) == 0)
				){

				endptr = last2en;
			}else{
				endptr = lasten;
			}
		}else{
			endptr = lasten;
		}
	}else{
		endptr = lasten;
	}

	*lasten = TEXT( '\\' );
	if( last2en )
		*last2en = TEXT( '\\' );
	if( last3en )
		*last3en = TEXT( '\\' );

	int leng;
	ZeroMemory( g_basedir, sizeof( WCHAR ) * MAX_PATH );
	leng = (int)( endptr - filename + 1 );
	wcsncpy_s( g_basedir, MAX_PATH, filename, leng );

	DbgOut( L"SetBaseDir : %s\r\n", g_basedir );

	return 0;
}

int OpenMQOFile()
{
	if( !g_texbank ){
		g_texbank = new CTexBank( s_pdev );
		if( !g_texbank ){
			_ASSERT( 0 );
			return 1;
		}
	}
//	if( s_model ){
//		delete s_model;
//		s_model = 0;
//		s_curboneno = -1;
//	}
	DestroyTimeLine( 1 );


	WCHAR buf[MAX_PATH];
	OPENFILENAME ofn;
	buf[0] = 0L;
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = NULL;
	ofn.hInstance = 0;
	ofn.lpstrFilter = L"mikoto File (*.mqo)\0*.mqo\0";
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter = 0;
	ofn.nFilterIndex = 1;
	ofn.lpstrFile = buf;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.lpstrTitle = NULL;
	ofn.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
	ofn.nFileOffset = 0;
	ofn.nFileExtension = 0;
	ofn.lpstrDefExt =NULL;
	ofn.lCustData = NULL;
	ofn.lpfnHook = NULL;
	ofn.lpTemplateName = NULL;
	if( GetOpenFileNameW(&ofn) == 0 ){
		return 0;
	}


    // Load the mesh
	s_model = new CModel();
	if( !s_model ){
		_ASSERT( 0 );
		return 1;
	}
	CallF( s_model->LoadMQO( s_pdev, buf, 0 ), return 1 );

	CallF( s_model->CalcInf(), return 1 );
	CallF( s_model->MakeDispObj(), return 1 );
	CallF( s_model->DbgDump(), return 1 );

	int mindex;
	mindex = s_modelindex.size();
	MODELELEM modelelem;
	modelelem.modelptr = s_model;
	modelelem.motmenuindex = 0;
	s_modelindex.push_back( modelelem );

    CDXUTComboBox* pComboBox = g_SampleUI.GetComboBox( IDC_COMBO_BONE );
	pComboBox->RemoveAllItems();

	map<int, CBone*>::iterator itrbone;
	for( itrbone = s_model->m_bonelist.begin(); itrbone != s_model->m_bonelist.end(); itrbone++ ){
		ULONG boneno = (ULONG)itrbone->first;
		CBone* curbone = itrbone->second;
		if( curbone && (boneno >= 0) ){
			char* nameptr = curbone->m_bonename;
			WCHAR wname[256];
			MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, nameptr, 256, wname, 256 );
			pComboBox->AddItem( wname, ULongToPtr( boneno ) );
		}
	}

	s_totalmb.center = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	s_totalmb.max = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	s_totalmb.min = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	s_totalmb.r = 0.0f;

	vector<MODELELEM>::iterator itrmodel;
	for( itrmodel = s_modelindex.begin(); itrmodel != s_modelindex.end(); itrmodel++ ){
		CModel* curmodel = itrmodel->modelptr;
		MODELBOUND mb;
		curmodel->GetModelBound( &mb );
		AddModelBound( &s_totalmb, &mb );
	}

    FLOAT fObjectRadius;
	g_vCenter = s_totalmb.center;
	fObjectRadius = s_totalmb.r;

    D3DXMatrixTranslation( &g_mCenterWorld, -g_vCenter.x, -g_vCenter.y, -g_vCenter.z );

    for( int i = 0; i < MAX_LIGHTS; i++ )
		g_LightControl[i].SetRadius( fObjectRadius );


    D3DXVECTOR3 vecEye( 0.0f, 0.0f, -15.0f );
    D3DXVECTOR3 vecAt ( 0.0f, 0.0f, -0.0f );
    g_Camera.SetViewParams( &vecEye, &vecAt );
    g_Camera.SetRadius( fObjectRadius * 3.0f, fObjectRadius * 0.5f, fObjectRadius * 10.0f );

	CallF( AddMotion(), return 1 );

	s_modelindex[ mindex ].tlarray = s_tlarray;
	s_modelindex[ mindex ].lineno2boneno = s_lineno2boneno;
	s_modelindex[ mindex ].boneno2lineno = s_boneno2lineno;

	CallF( OnModelMenu( mindex ), return 1 );

	return 0;
}

int AddTimeLine( WCHAR* newname, int newmotid )
{
	//EraseKeyList();

	if( !s_owpTimeline ){
		OWP_Timeline* owpTimeline = 0;
		//^CCGUIp[c𐶐
		owpTimeline= new OWP_Timeline( L"testmotion" );

		//EBhEɃ^CC֘At
		s_timelineWnd->addParts(*owpTimeline);

		// J[\ړ̃CxgXi[
		// J[\ړtOcursorFlagIɂ郉_֐o^
		owpTimeline->setCursorListener( [](){ s_cursorFlag=true; } );

		// L[ĨCxgXi[
		// L[ItOselectFlagIɂ郉_֐o^
		owpTimeline->setSelectListener( [](){ s_selectFlag=true; } );


		// L[ړ̃CxgXi[
		// L[ړtOkeyShiftFlagIɂāAL[ړʂRs[郉_֐o^
		owpTimeline->setKeyShiftListener( [](){
			s_keyShiftFlag= true;
			s_keyShiftTime= s_owpTimeline->getShiftKeyTime();
		} );

		// L[폜̃CxgXi[
		// 폜ꂽL[X^bN郉_֐o^
		owpTimeline->setKeyDeleteListener( [](const OWP_Timeline::KeyInfo &keyInfo){
			s_deletedKeyInfoList.push_back(keyInfo);
		} );

		s_owpTimeline = owpTimeline;
	}

	int nextindex;
	nextindex = s_tlarray.size();

	TLELEM newtle;
	newtle.menuindex = nextindex;
	newtle.motionid = newmotid;
	s_tlarray.push_back( newtle );


	CallF( s_model->SetCurrentMotion( newmotid ), return 1 );


	//^CC̃L[ݒ
	refreshTimeline(*s_owpTimeline);

	return 0;
}

//^CCɃ[Vf[^̃L[ݒ肷
void refreshTimeline(OWP_Timeline& timeline){

	//ݒ
	if( s_model ){
		timeline.setMaxTime( s_model->m_curmotinfo->frameleng );
	}
	//ׂĂ̍sNA
	timeline.deleteLine();

	s_lineno2boneno.erase( s_lineno2boneno.begin(), s_lineno2boneno.end() );
	s_boneno2lineno.erase( s_boneno2lineno.begin(), s_boneno2lineno.end() );

	if( s_model ){
		CallF( s_model->FillTimeLine( timeline, s_lineno2boneno, s_boneno2lineno ), return );
	}else{
		WCHAR label[256];
		swprintf_s(label, 256, L"dummy%d", 0 );
		timeline.newLine(label);
	}
	//Iݒ
	timeline.setCurrentLine( 0 );
	timeline.setCurrentTime(0.0);
}

int AddBoneEul( float addx, float addy, float addz )
{
	if( !s_model || (s_curboneno < 0) ){
		return 0;
	}

	D3DXVECTOR3 cureul, neweul;

	CMotionPoint* newmp = 0;
	int existflag = 0;
	int calcflag = 1;
	_ASSERT( s_model->m_curmotinfo );
	newmp = s_model->AddMotionPoint( s_model->m_curmotinfo->motid, s_owpTimeline->getCurrentTime(), s_curboneno, calcflag, &existflag );
	if( !newmp ){
		_ASSERT( 0 );
		return 1;
	}

	cureul = newmp->m_eul;
	neweul.x = cureul.x + addx;
	neweul.y = cureul.y + addy;
	neweul.z = cureul.z + addz;
	newmp->SetEul( neweul );

	if( existflag == 0 ){
		CBone* curbone = s_model->m_bonelist[ s_curboneno ];
		_ASSERT( curbone );
		s_owpTimeline->newKey( curbone->m_wbonename, s_owpTimeline->getCurrentTime(), (void*)newmp );
	}

	return 0;
}

int AddBoneTra( float addx, float addy, float addz )
{
	if( !s_model || (s_curboneno < 0) ){
		return 0;
	}

	D3DXVECTOR3 curtra, newtra;

	CMotionPoint* newmp = 0;
	int existflag = 0;
	int calcflag = 1;
	_ASSERT( s_model->m_curmotinfo );
	newmp = s_model->AddMotionPoint( s_model->m_curmotinfo->motid, s_owpTimeline->getCurrentTime(), s_curboneno, calcflag, &existflag );
	if( !newmp ){
		_ASSERT( 0 );
		return 1;
	}

	curtra = newmp->m_tra;
	newtra.x = curtra.x + addx;
	newtra.y = curtra.y + addy;
	newtra.z = curtra.z + addz;
	newmp->m_tra = newtra;

	if( existflag == 0 ){
		CBone* curbone = s_model->m_bonelist[ s_curboneno ];
		_ASSERT( curbone );
		s_owpTimeline->newKey( curbone->m_wbonename, s_owpTimeline->getCurrentTime(), (void*)newmp );
	}

	return 0;
}

int DispMotionWindow()
{
	if( !s_timelineWnd ){
		return 0;
	}

	if( s_dispmw ){
		s_timelineWnd->setVisible( false );
		s_dispmw = false;
	}else{
		s_timelineWnd->setVisible( true );
		s_dispmw = true;
	}

	return 0;
}
int DispToolWindow()
{
	if( !s_toolWnd ){
		return 0;
	}

	if( s_disptool ){
		s_toolWnd->setVisible( false );
		s_disptool = false;
	}else{
		s_toolWnd->setVisible( true );
		s_disptool = true;
	}

	return 0;
}
int DispObjPanel()
{
	if( !s_layerWnd ){
		return 0;
	}

	if( s_dispobj ){
		s_layerWnd->setVisible( false );
		s_dispobj = false;
	}else{
		s_layerWnd->setVisible( true );
		s_dispobj = true;
	}

	return 0;
}


int EraseKeyList()
{
	s_copyKeyInfoList.erase( s_copyKeyInfoList.begin(), s_copyKeyInfoList.end() );
	s_selectKeyInfoList.erase( s_selectKeyInfoList.begin(), s_selectKeyInfoList.end() );
	s_deletedKeyInfoList.erase( s_deletedKeyInfoList.begin(), s_deletedKeyInfoList.end() );

	return 0;
}

int AddMotion()
{
	int motnum = s_tlarray.size();
	if( motnum >= MAXMOTIONNUM ){
		MessageBox( s_mainwnd, L"ȏヂ[Vǂݍ߂܂B", L"[V܂B", MB_OK );
		return 0;
	}

	char motionname[256];
	ZeroMemory( motionname, 256 );
	SYSTEMTIME systime;
	GetLocalTime( &systime );
	sprintf_s( motionname, 256, "motion_%d_%d_%d_%d_%d_%d_%d",
		systime.wYear,
		systime.wMonth,
		systime.wDay,
		systime.wHour,
		systime.wMinute,
		systime.wSecond,
		systime.wMilliseconds
	);

	int newmotid = -1;
	double motleng = 100.0;
	CallF( s_model->AddMotion( motionname, motleng, &newmotid ), return 1 );

	WCHAR wmotionname[256];
	MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, motionname, 256, wmotionname, 256 );

	CallF( AddTimeLine( wmotionname, newmotid ), return 1 );

	int selindex = s_tlarray.size() - 1;
	CallF( OnAnimMenu( selindex ), return 1 );


	return 0;
}

int OnAnimMenu( int selindex )
{
	s_curmotmenuindex = selindex;

	if( selindex < 0 ){
		return 0;//!!!!!!!!!
	}

	_ASSERT( s_animmenu );

	int iAnimSet, cAnimSets;
	cAnimSets = GetMenuItemCount( s_animmenu );
	for (iAnimSet = 0; iAnimSet < cAnimSets; iAnimSet++)
	{
		RemoveMenu(s_animmenu, 0, MF_BYPOSITION);
	}


	if( !s_model ){
		return 0;//!!!!!!!!!!!!!!!!!!
	}

	cAnimSets = s_tlarray.size();

	if( cAnimSets <= 0 ){
		return 0;//!!!!!!!!!!!!!!!!!!!
	}

	char* szName;
	WCHAR wname[256];
	for(iAnimSet = 0; iAnimSet < cAnimSets; iAnimSet++)
	{
		int motid;
		motid = s_tlarray[iAnimSet].motionid;
		szName = s_model->m_motinfo[ motid ]->motname;
		MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, szName, 256, wname, 256 );

		if (szName != NULL)
			AppendMenu(s_animmenu, MF_STRING, 59900 + iAnimSet, wname);
		else
			AppendMenu(s_animmenu, MF_STRING, 59900 + iAnimSet, L"<No Animation Name>");
	}

	if( cAnimSets > 0 ){
		CheckMenuItem(s_mainmenu, 59900 + selindex, MF_CHECKED);

		int selmotid;
		selmotid = s_tlarray[ selindex ].motionid;
		if( selmotid > 0 ){
			CallF( s_model->SetCurrentMotion( selmotid ), return 1 );
			//EraseKeyList();
			s_owpTimeline->setCurrentLine( 0 );
			s_owpTimeline->setCurrentTime( 0.0 );

			//^CC̃L[ݒ
			refreshTimeline(*s_owpTimeline);
			s_owpTimeline->setCurrentTime( 0.0 );
		}

	}
	return 0;
}

int OnModelMenu( int selindex )
{
	s_curmodelmenuindex = selindex;

	_ASSERT( s_modelmenu );
	int iMdlSet, cMdlSets;
	cMdlSets = GetMenuItemCount( s_modelmenu );
	for (iMdlSet = 0; iMdlSet < cMdlSets; iMdlSet++)
	{
		RemoveMenu(s_modelmenu, 0, MF_BYPOSITION);
	}

	if( (selindex < 0) || !s_model ){
		s_model = 0;
		s_curboneno = -1;
		if( s_owpTimeline ){
			refreshTimeline(*s_owpTimeline);
		}
		refreshModelPanel();
		return 0;//!!!!!!!!!
	}

	cMdlSets = s_modelindex.size();
	if( cMdlSets <= 0 ){
		s_model = 0;
		if( s_owpTimeline ){
			refreshTimeline(*s_owpTimeline);
		}
		refreshModelPanel();
		return 0;//!!!!!!!!!!!!!!!!!!!
	}

	WCHAR* wname;
	for(iMdlSet = 0; iMdlSet < cMdlSets; iMdlSet++)
	{
		wname = s_modelindex[iMdlSet].modelptr->m_filename;

		if( *wname != 0 )
			AppendMenu(s_modelmenu, MF_STRING, 61000 + iMdlSet, wname);
		else
			AppendMenu(s_modelmenu, MF_STRING, 61000 + iMdlSet, L"<No Model Name>");
	}

	if( cMdlSets > 0 ){
		CheckMenuItem( s_mainmenu, 61000 + selindex, MF_CHECKED );

		s_model = s_modelindex[ selindex ].modelptr;
		s_tlarray = s_modelindex[ selindex ].tlarray;
		s_curmotmenuindex = s_modelindex[ selindex ].motmenuindex;
		s_lineno2boneno = s_modelindex[ selindex ].lineno2boneno;
		s_boneno2lineno = s_modelindex[ selindex ].boneno2lineno;

		OnAnimMenu( s_curmotmenuindex );
	}

	refreshModelPanel();

	return 0;
}



int OnDelMotion( int delmenuindex )
{
	int tlnum = s_tlarray.size();
	if( (tlnum <= 0) || (delmenuindex < 0) || (delmenuindex >= tlnum) ){
		return 0;
	}

	int delmotid = s_tlarray[ delmenuindex ].motionid;
	CallF( s_model->DeleteMotion( delmotid ), return 1 );


	int tlno;
	for( tlno = delmenuindex; tlno < (tlnum - 1); tlno++ ){
		s_tlarray[ tlno ] = s_tlarray[ tlno + 1 ];
	}
	s_tlarray.pop_back();

	int newtlnum = s_tlarray.size();
	if( newtlnum == 0 ){
		AddMotion();
	}else{
		OnAnimMenu( 0 );
	}

	return 0;
}

int OnDelModel( int delmenuindex )
{
	int mdlnum = s_modelindex.size();
	if( (mdlnum <= 0) || (delmenuindex < 0) || (delmenuindex >= mdlnum) ){
		return 0;
	}

	CModel* delmodel = s_modelindex[ delmenuindex ].modelptr;
	if( delmodel ){
		delete delmodel;
	}

	int mdlno;
	for( mdlno = delmenuindex; mdlno < (mdlnum - 1); mdlno++ ){
		s_modelindex[ mdlno ].tlarray.erase( s_modelindex[ mdlno ].tlarray.begin(), s_modelindex[ mdlno ].tlarray.end() );
		s_modelindex[ mdlno ].boneno2lineno.erase( s_modelindex[ mdlno ].boneno2lineno.begin(), s_modelindex[ mdlno ].boneno2lineno.end() );
		s_modelindex[ mdlno ].lineno2boneno.erase( s_modelindex[ mdlno ].lineno2boneno.begin(), s_modelindex[ mdlno ].lineno2boneno.end() );

		s_modelindex[ mdlno ] = s_modelindex[ mdlno + 1 ];
	}
	s_modelindex.pop_back();

	if( s_modelindex.empty() ){
		s_curboneno = -1;
		s_model = 0;
		s_curmodelmenuindex = -1;
		s_tlarray.erase( s_tlarray.begin(), s_tlarray.end() );
		s_curmotmenuindex = -1;
		s_lineno2boneno.erase( s_lineno2boneno.begin(), s_lineno2boneno.end() );
		s_boneno2lineno.erase( s_boneno2lineno.begin(), s_boneno2lineno.end() );
	}else{
		s_curboneno = -1;
		s_model = s_modelindex[ 0 ].modelptr;
		s_tlarray = s_modelindex[ 0 ].tlarray;
		s_curmotmenuindex = s_modelindex[ 0 ].motmenuindex;
		s_lineno2boneno = s_modelindex[ 0 ].lineno2boneno;
		s_boneno2lineno = s_modelindex[ 0 ].boneno2lineno;
	}

	OnModelMenu( 0 );


	return 0;
}

int OnDelAllModel()
{
	int mdlnum = s_modelindex.size();
	if( mdlnum <= 0 ){
		return 0;
	}

	vector<MODELELEM>::iterator itrmodel;
	for( itrmodel = s_modelindex.begin(); itrmodel != s_modelindex.end(); itrmodel++ ){
		CModel* delmodel = itrmodel->modelptr;
		if( delmodel ){
			delete delmodel;
		}
		itrmodel->tlarray.erase( itrmodel->tlarray.begin(), itrmodel->tlarray.end() );
		itrmodel->boneno2lineno.erase( itrmodel->boneno2lineno.begin(), itrmodel->boneno2lineno.end() );
		itrmodel->lineno2boneno.erase( itrmodel->lineno2boneno.begin(), itrmodel->lineno2boneno.end() );
	}

	s_modelindex.erase( s_modelindex.begin(), s_modelindex.end() );

	s_curboneno = -1;
	s_model = 0;
	s_curmodelmenuindex = -1;
	s_tlarray.erase( s_tlarray.begin(), s_tlarray.end() );
	s_curmotmenuindex = -1;
	s_lineno2boneno.erase( s_lineno2boneno.begin(), s_lineno2boneno.end() );
	s_boneno2lineno.erase( s_boneno2lineno.begin(), s_boneno2lineno.end() );

	OnModelMenu( -1 );

	return 0;
}



int AddModelBound( MODELBOUND* mb, MODELBOUND* addmb )
{
	D3DXVECTOR3 newmin = mb->min;
	D3DXVECTOR3 newmax = mb->max;

	if( newmin.x > addmb->min.x ){
		newmin.x = addmb->min.x;
	}
	if( newmin.y > addmb->min.y ){
		newmin.y = addmb->min.y;
	}
	if( newmin.z > addmb->min.z ){
		newmin.z = addmb->min.z;
	}

	if( newmax.x < addmb->max.x ){
		newmax.x = addmb->max.x;
	}
	if( newmax.y < addmb->max.y ){
		newmax.y = addmb->max.y;
	}
	if( newmax.z < addmb->max.z ){
		newmax.z = addmb->max.z;
	}

	mb->center = ( newmin + newmax ) * 0.5f;
	mb->min = newmin;
	mb->max = newmax;

	D3DXVECTOR3 diff;
	diff = mb->center - newmin;
	mb->r = D3DXVec3Length( &diff );

	return 0;
}

int refreshModelPanel()
{
	//ׂĂ̍sNA
	s_owpLayerTable->deleteLine();

	WCHAR label[256];
	int objnum = 0;

	if( s_model ){
		objnum = s_model->m_object.size();
	}else{
		objnum = 0;
	}
	
	if( objnum > 0 ){
		map<int, CMQOObject*>::iterator itrobj;
		for( itrobj = s_model->m_object.begin(); itrobj != s_model->m_object.end(); itrobj++ ){
			CMQOObject* curobj = itrobj->second;
			WCHAR wname[256];
			MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, curobj->m_name, 256, wname, 256 );
			s_owpLayerTable->newLine( wname, (void*)curobj );
			s_owpLayerTable->setVisible( wname, (curobj->m_dispflag > 0), true );
		}
	}else{
		wcscpy_s( label, 256, L"dummy name" );
		s_owpLayerTable->newLine( label, 0 );
	}

	return 0;
}